

#include <iomanip>
#include <iostream>
#include <DataFilter.h>
#include <evselect.h>
#include <ParameterInfo.h>
#include <ExceptionErrorHandler.h>
#include <cmath>
#include "Xmm.h"

#include <unistd.h>

#include "stringutil.h"
#include "LightCurve.h"
#include "FrameTable.h"
#include "GtiCorr.h"


LightCurve::LightCurve(string timeSerie, string evtList)
{
	_bkgCorrected = false;
	_eventsMap = new std::multimap<double,int>;

	_LC = setLC(timeSerie);
	_LC->_name = timeSerie;
	setDSS();
	closeTimeSerie();
	setEventList(evtList);
	setCCDsInRegion();
	closeEventList();
	if (_ccdInRegion.size() >1 )
	{
		// The original Time Serie has not got any CCD attached.
		// So, we set the 0 to this particular Time Serie.
		_LC->setCCDNumber(0);
		// We only recalculate the CCDs time series for those cases where
		// the selection region overlap more than one CCD
		createCCDsLC();
	}
	else
		_LC->setCCDNumber(_ccdInRegion[0]);

}

LightCurve::LightCurve(string timeSerie)
{
	setLC(timeSerie);
	setDSS();
	closeTimeSerie();
	_evtExists = false;
	_bkgCorrected = false;

	_eventsMap = new std::multimap<double,int>;
}

LightCurve::LightCurve(double val0, double val1, double binSize)
{
	errstr << "creating a simple light curve" << Sas::message(Msg::App,Msg::Sparse);
	_LC = new LCGrid(val0,val1,binSize);
	_tsExists = false;
	_evtExists = false;
	_bkgCorrected = false;

	_eventsMap = new std::multimap<double,int>;
}

LightCurve::~LightCurve()
{
	errstr << "LightCurve Destructor "<< Sas::message(Msg::App,Msg::Sparse);
	delete _LC;

	if (_bkgCorrected)
		delete _spec;

	for (unsigned int i = 0 ; i < _ccdLC.size() ; i++)
	{
		dataSetServer->remove(_ccdLC[i]->_name);
		delete _ccdLC[i];
	}

	errstr << "Deleting events Map "<< Sas::message(Msg::App,Msg::Sparse);
	delete _eventsMap;


	errstr << "Done "<< Sas::message(Msg::App,Msg::Sparse);
}

LCGrid* LightCurve::setLC(string timeSerie)
{
	_tsExists = true;
	// Read the Time serie file
	_ts =  dataSetServer->open(timeSerie, Dal::Read);

	// Read from the Time Serie header the First bin, the lastest bin and
	// the bin size.
	// We cannot use the TSTART and the TSTOP keywords, because they are
	// the time stamp of the first and the latest event or the timemin and timemax values of
	// evselect parameters.
	// TIMEDEL == Light curve bin size

	Table *tsTab = _ts->table("RATE");

	// Set the INSTRUMENT and TIME keywords attribute
	_instrument = tsTab->attribute("INSTRUME")->asString();
	_filter = tsTab->attribute("FILTER")->asString();
	_telescope = tsTab->attribute("TELESCOP")->asString();
	_mjdref = tsTab->attribute("MJDREF")->asReal();
	_timeSys = tsTab->attribute("TIMESYS")->asString();
	_timeSysComment = tsTab->attribute("TIMESYS")->label();
	_timeUnit = tsTab->attribute("TIMEUNIT")->asString();
	_timeRef = tsTab->attribute("TIMEREF")->asString();
	_timeRefComment = tsTab->attribute("TIMEREF")->label();
	_timeAssign = tsTab->attribute("TASSIGN")->asString();
	_timeAssignComment = tsTab->attribute("TASSIGN")->label();

	_timeClockApp = tsTab->attribute("CLOCKAPP")->asString();

	ColumnData *timeDat = tsTab->column("TIME")->data();
	real64 *time = timeDat->real64Data();


	double binSize = tsTab->attribute("TIMEDEL")->asReal();
	// LCGrid needs as input the start time of the time serie, the stop time and the bin size.
	// Remember that the TIME column is the time bin middle value.
	LCGrid *lc = new LCGrid(time[0]-(binSize/2.0),time[tsTab->rows()-1]+(binSize/2.0),binSize);

	// Store the RATE or the COUNT values

	string colName = tsTab->hasColumn("RATE") ? "RATE" : "COUNTS";
	ColumnData *rateDat = tsTab->column(colName)->data();
	if ( colName == "RATE" )
	{
		real64 *rate = rateDat->real64Data();
		lc->setCounts(rate);
	}
	else // Assume COUNTS column
	{
		int32 *counts = rateDat->int32Data();

		lc->setCounts(counts);
	}

	// Setting the DataSubspace;

	delete rateDat;
	delete timeDat;

	return lc;

}

void LightCurve::setDSS()
{
	Block * eblk = _ts->block(_ts->name()+":RATE");
	eblk >> _dss;

}

void LightCurve::setEventList(string evtList)
{
	_evtExists = true;
	_evtListName = evtList;
	dataSetServer->copy(evtList, "tmp.ds");
	_evtList = dataSetServer->open("tmp.ds", Dal::Modify);
}

void LightCurve::setCCDsInRegion()
{

	Table *srcTable = _evtList->table("EVENTS");

	//Source filtering

	DataFilter filter(_dss.selectExpression());
	unsigned long selectedEvents = filter.doFilter(*srcTable);

	_subMode = srcTable->attribute("SUBMODE")->asString();

	errstr << "selected " << selectedEvents << " rows from the source input table."
	  	     << Sas::message(Msg::App,Msg::Sparse);

	ColumnData * cCcdDat = srcTable->column("CCDNR")->data();
	int8 * ccdDat = cCcdDat->int8Data();
	ColumnData *cTimeDat = srcTable->column("TIME")->data();
	real64 *timeDat = cTimeDat->real64Data();


	unsigned long mRows = srcTable->rows();

	int ccdNum=0;
	for(unsigned long irow=0; irow<mRows; irow++){
		if (ccdNum != ccdDat[irow])
			_ccdInRegion.push_back(ccdDat[irow]);

		ccdNum = ccdDat[irow];
		_eventsMap->insert(make_pair(timeDat[irow],ccdDat[irow]));
	}

	delete cCcdDat;
	delete cTimeDat;
}

void LightCurve::createCCDsLC()
{
	for (unsigned int i = 0 ; i < _ccdInRegion.size() ; i++)
	{
		// Create a lightcurve for only one CCD
		std::stringstream o;
		o << "( "<<_dss.selectExpression() << ") ";
		o << "&& (CCDNR == "<< _ccdInRegion[i] << ") ";
		string expression = o.str();
		Info::Task* evselect = task("evselect");
		ParameterInfo& evselectpars = *evselect->parameters();
		evselectpars.revertToDefault();

		evselectpars["table"]->setValue(_evtListName+ ":EVENTS");
		evselectpars["expression"]->setValue(expression);
		evselectpars["writedss"]->setValue(true);
		evselectpars["updateexposure"]->setValue(false);
		evselectpars["withrateset"]->setValue(true);
		std::stringstream ccd;
		ccd << "rate_ccd" << _ccdInRegion[i]<<"_"<<_LC->_name <<".fits";
		evselectpars["rateset"]->setValue(ccd.str());
		evselectpars["timebinsize"]->setValue(_LC->_stride);
		evselectpars["withtimeranges"]->setValue(true);
		evselectpars["timemin"]->setValue(_LC->_start);
		evselectpars["timemax"]->setValue(_LC->_stop);
		evselectpars["maketimecolumn"]->setValue(true);
		evselectpars["makeratecolumn"]->setValue(true);
		try{
		    evselect->exec(true);
		    _ccdLC.push_back(setLC(ccd.str()));
		    // Set the CCD where this LC belongs;
		    _ccdLC[i]->setCCDNumber(_ccdInRegion[i]);
		    _ccdLC[i]->_name = ccd.str();
		  }
		  catch (SasException::Error e) {
		    errstr << evselect->name() << " terminated with the error " << e.code
			   << " that said: \n" << e.message
			   << Sas::warning("SubTaskError");

		  }
		  catch (SasException::Fatal e) {
		    errstr << evselect->name() << " terminated with the fatal error " << e.code
			   << " that said: \n" << e.message
			   << Sas::warning("SubTaskError");
		  }
	}
}

void LightCurve::doRelativeCorr()
{
	//Check if we have more than one CCD in the selection region.

	if (_ccdInRegion.size() > 1)
	{
		errstr << "More than one CCD inside the extraction region" <<
		Sas::message(Msg::App,Msg::Sparse);
		// Check that we have the sum of the CCDs LC Counts is the same as the
		// Counts in the original CCD

		std::vector<double> ccdCountVal =  _LC->getCounts();
		for (unsigned int i = 0 ; i < _ccdInRegion.size() ; i++)
			ccdCountVal = ccdCountVal - _ccdLC[i]->getCounts();

		for (unsigned int i = 0 ; i < ccdCountVal.size() ; i++)
		{
			if ( ccdCountVal[i] >= 0.5 )
			{
				errstr << "A difference in the sum of the counts of single CCDs " <<
						 "with respect to the counts of the original light curve has been found.  "
						 << Sas::warning("CCDs Count Difference");
			}
		}

		// Apply the relative correction for each CCDs Light Curves
		errstr << "Calculating the relative corrections for each CCD..." <<
				Sas::message(Msg::App,Msg::Sparse);

		for (unsigned int i = 0 ; i < _ccdInRegion.size() ; i++)
		{
			errstr << "Correction for CCD: "<< _ccdLC[i]->_ccd <<
					Sas::message(Msg::App,Msg::Sparse);
			doGTICorr(_ccdLC[i]);
			doExposureCorr(_ccdLC[i]);
		}
		errstr << "Done." << Sas::message(Msg::App,Msg::Sparse);

		// Calculate the weigth that we will use afterwards to calculate the
		// FracExp of the corresponding original Light Curve.
		errstr << "Calculating the fracExp weight for each CCD..." <<
				Sas::message(Msg::App,Msg::Sparse);
		for (unsigned int i = 0 ; i < _ccdLC.size() ; i++)
			_ccdLC[i]->calFracExpWeight();


		errstr << "Done." << Sas::message(Msg::App,Msg::Sparse);

		// Add the corresponding FracExp Weight of each CCD Light Curve
		errstr << "Calculating the final fracExp value..." <<
				Sas::message(Msg::App,Msg::Sparse);

		std::vector<double> fracExpWeight = _ccdLC[0]->getFracExpWeight();


		for (unsigned int i = 1 ; i < _ccdLC.size() ; i++)
			fracExpWeight = fracExpWeight + _ccdLC[i]->getFracExpWeight();

		// Multiply the Weight by the original number of counts
		std::vector<double> fracExp;
		fracExp = _LC->getCounts() / fracExpWeight;

		errstr << "Done." << Sas::message(Msg::App,Msg::Sparse);

		// Set the new FracExp to the original LC
		_LC->setFracExp(fracExp);

	}
	else
	{
		errstr << "Extraction region lies in CCD number " << _LC->getCCDNumber()
			<< Sas::message(Msg::App,Msg::Sparse);
		errstr << "Calculating the relative corrections for each CCD..." <<
				Sas::message(Msg::App,Msg::Sparse);

		doGTICorr(_LC);
		doExposureCorr(_LC);
		errstr << "Relative Corrections Done." << Sas::message(Msg::App,Msg::Sparse);

	}

}


void LightCurve::doGTICorr(LCGrid *lc)
{
	std::vector<std::string> nameExtVec = searchFor(lc);

	// Set up GTI info
	// Combine all the GTI extension of the Time Serie
	errstr << "Doing GTI Correction..."<< Sas::message(Msg::App,Msg::Sparse);
	// Loop over all the GTI extension and combine all into a single
	// OGIPgtiTable.
	GtiList li1;

	for (unsigned int i = 0 ; i < nameExtVec.size() ; i++)
	{
		errstr << "GTI Extension["<<nameExtVec[i] <<"]"
			<< Sas::message(Msg::App,Msg::Sparse);

		InOGIPgtiTable in1 = InOGIPgtiTable(lc->_name+":"+nameExtVec[i]);
		GtiList li; in1>>li;
		if (i == 0)
			li1 = li;
		else
			li1 |= li;
	}

	li1.sortIt();

	GtiCorr gti(li1);
	gti.doCorr(lc);
	gti.printCorr();
	errstr << "Done." << Sas::message(Msg::App,Msg::Sparse);

}


void LightCurve::doExposureCorr(LCGrid *lc)
{
	std::stringstream ccd;
	bool randFlag;
	ccd << ":EXPOSU";
	lc->getCCDNumber() < 10 ? ccd <<"0"<<lc->getCCDNumber() : ccd << lc->getCCDNumber();

	errstr  << "Doing Exposure Correction "
	<< _evtListName+ccd.str() <<  Sas::message(Msg::App,Msg::Sparse);

	FrameTable in2(_evtListName+ccd.str());
	FrameList li2; in2>>li2;
	/*
	 * Check if the events have been randomize
	 */
	randFlag = in2.getRandFlag();

	if (randFlag)
		errstr  << "Events already randomized..."<<  Sas::message(Msg::App,Msg::Sparse);


	in2.closeDataSet();

	Exposure exposure(li2);
	exposure.doCorr(lc, _eventsMap, _instrument,randFlag,_subMode);
	exposure.printCorr();
	errstr << "Exposure Correction Done." << Sas::message(Msg::App,Msg::Sparse);


	/*
	 * Dead Time Correction
	 */

	errstr  << "Doing Dead Time Correction "<<  Sas::message(Msg::App,Msg::Sparse);
	DeadTime deadTime(li2);
	deadTime.doCorr(lc);
	deadTime.printCorr();
	errstr << "Dead Time Correction Done." << Sas::message(Msg::App,Msg::Sparse);
	/*
	 * Dead Time Correction
	 */

}


std::vector<std::string> LightCurve::searchFor(LCGrid *lc)
{
	std::vector<std::string> values;
	DataSet *ts  =  dataSetServer->open(lc->_name, Dal::Read);
	DataSubspace dss;
	Block * eblk = ts->block(ts->name()+":RATE");
	eblk >> dss;


	unsigned ncomps = dss.components();

	for(unsigned icomp=0; icomp<ncomps; icomp++)
	{
		DScomp * comp = dss.component(icomp);
		for(unsigned ifilt=0; ifilt<comp->filters(); ifilt++){
			DSfilter * filter = comp->filter(ifilt);

			if( (filter->type()==DSS::GtiT)	 )
    	    {
    	    	std::vector<std::string> vec(breakup(filter->blockSpec(), ":"));
    	    	// Check if we have a STDGTI block or a GTI block
    	    	std::stringstream ss;
    	    	double number;
    	    	if (vec[1].find("STDGTI") != string::npos)
    	    	{
    	    		errstr  << vec[1] <<" extension found "<<  Sas::message(Msg::App,Msg::Sparse);
        	    	ss << vec[1][vec[1].size()-2] << vec[1][vec[1].size()-1];
        	    	ss >> number;
    	    	}
    	    	else
    	    	{
    	    		errstr  << vec[1] <<" extension found "<<  Sas::message(Msg::App,Msg::Sparse);
        	    	ss << vec[1][vec[1].size()-5] << vec[1][vec[1].size()-4];
        	    	ss >> number;

        	    	// I do not know why, with GTI extensions, the ccd number is decreased by 1.
        	    	// New dsslib version 5.6 -> fixes this problem that has to do with the
        	    	// GTI naming. Now, all the GTI extensions has its own CCD number in the table
        	    	// name, then it is simply to identify the GTI that you have to use.
        	    	// AI: Fixes SOC-SPR-6503
        	    	//number++;
    	    	}

                if (number == lc->getCCDNumber())
                {
                	errstr  <<"Processing GTI extension: "<< vec[1] <<  Sas::message(Msg::App,Msg::Sparse);
					values.push_back(vec[1]);
                }

    	    }
		}
	}

	dataSetServer->close(ts);

	if (values.size() == 0)
		errstr << "No GTI found for the CCD number "<< lc->getCCDNumber()
			<< Sas::error("EmptyGTIList");

	return values;
}

void LightCurve::closeTimeSerie()
{
	if (_tsExists == true )
		dataSetServer->close(_ts);
}

void LightCurve::closeEventList()
{
	errstr << "Closing event file dataset"<< Sas::message(Msg::App,Msg::Sparse);
	if (_evtExists == true)
	{
		dataSetServer->close(_evtList);
		unlink("tmp.ds");
	}
}
LCGrid* LightCurve::getLC()
{
	return _LC;
}

void LightCurve::calSpec()
{
	errstr << "Calculating spectrum..."<< Sas::message(Msg::App,Msg::Sparse);
	_bkgCorrected = true;

	// Creates an spectrum. Store all the information to be used for the
	// absolute correction.
	Info::Task* evselect = task("evselect");
	ParameterInfo& evselectpars = *evselect->parameters();
	evselectpars.revertToDefault();

	evselectpars["table"]->setValue(_evtListName+ ":EVENTS");
	evselectpars["writedss"]->setValue(true);
	std::vector<std::string> vec = breakup(_LC->_name, "/");
	std::string name = vec[vec.size()-1];

	std::stringstream specName;
	specName << "spectrum_" << name;
	evselectpars["expression"]->setValue(_dss.selectExpression());
	evselectpars["energycolumn"]->setValue("PI");
	evselectpars["withspectrumset"]->setValue(true);
	evselectpars["spectrumset"]->setValue(specName.str());
	evselectpars["spectralbinsize"]->setValue(50);
	try{
	    evselect->exec(true);
	    // Create spectrum.
	    _spec = new Spec(specName.str());
	    _spec->setEventFileName(_evtListName);
	}
	catch (SasException::Error e) {
		errstr << evselect->name() << " terminated with the error " << e.code
		   << " that said: \n" << e.message
		   << Sas::warning("SubTaskError");

	}
	catch (SasException::Fatal e) {
		errstr << evselect->name() << " terminated with the fatal error " << e.code
		   << " that said: \n" << e.message
		   << Sas::warning("SubTaskError");
	}

	errstr << "Done."<< Sas::message(Msg::App,Msg::Sparse);
}

Spec* LightCurve::getSpec()
{
	return _spec;
}

void LightCurve::doAbsoluteCorr()
{
	/*
	 * Calculate the average of the off/on axis ARF, weighted with the spectrum
	 * counts.
	 */
	std::vector<double> ratio = _spec->_onSpecResp/_spec->_srcSpecResp;

	/*
	 * Convert the Channel into energy (keV)
	 */
	double energy;

	std::vector<double> absCorr;
	std::vector<double> weights;
	for (unsigned int i = 0 ; i < _spec->_channels.size() ; i++)
	{
		unsigned int energyCounter = 0;
		energy = ((i*_spec->_specdelt)+_spec->_specval)/1000.0;

		while ((!_spec->_energyList[energyCounter].inside(energy))
				&& (energyCounter < ratio.size()))
			energyCounter++;

		if (energyCounter <  ratio.size())
		{
			//cout << "CHANNEL "<< i << " ENERGYCOUNTER " << energyCounter+1
			//	<< " COUNTS " << _spec->_counts[i] << " RESP " << ratio[energyCounter+1] << endl;
			absCorr.push_back(_spec->_counts[i]*ratio[energyCounter+1]);
			weights.push_back(_spec->_counts[i]);
		}

	}


	/*
	 * Weight the ratio with the spectrum counts
	 */
	double val = 0.0;
	double w = 0.0;
	for (unsigned int i = 0 ; i < absCorr.size() ; i++)
	{
		val += absCorr[i];
		w += weights[i];
	}
	_spec->setWeigthedMean( val/w);

	errstr << "Weighted Mean "<< _spec->getWeigthedMean() << Sas::message(Msg::App,Msg::Sparse);
	if ( _spec->getWeigthedMean() == 0.0)
		errstr << "Zero Absolute Correction. Please check your source and background expression"
			<< Sas::error("ZeroAbsoluteCorrection");
}


void LightCurve::applyAbsCorr()
{
	errstr << "Applying Absolute Correction... "   << Sas::message(Msg::App,Msg::Sparse);
	//_LC->_counts = _LC->_counts * _spec->getWeigthedMean();

	_LC->_rateCorr = _LC->_rateCorr * _spec->getWeigthedMean();
	_LC->_errRateCorr = _LC->_errRateCorr * _spec->getWeigthedMean();
	errstr << "Done. "   << Sas::message(Msg::App,Msg::Sparse);
}

void LightCurve::createFile(string outFileName)
{
	DataSet *out  =  dataSetServer->open(outFileName, Dal::Create);
	Table *outTable = out->addTable("RATE",_LC->_timeStamp.size());

	// Create columns
	real64 *timeDat=0, *rateDat=0, *rateErrDat=0, *fracExpDat=0;
	ColumnData *cTimeDat=0, *cRateDat=0, *cRateErrDat=0, *cFracExpDat=0;
	cTimeDat = outTable->addColumn("TIME", Dal::Real64, " Time of the center of each frame", "s")->data();
	cRateDat = outTable->addColumn("RATE", Dal::Real64, "", "count/s")->data();
	cRateErrDat = outTable->addColumn("ERROR", Dal::Real64, "", "count/s")->data();
	cFracExpDat = outTable->addColumn("FRACEXP", Dal::Real64, "effective time / real time", "fraction")->data();

	timeDat = cTimeDat->real64Data();
	rateDat = cRateDat->real64Data();
	rateErrDat = cRateErrDat->real64Data();
	fracExpDat = cFracExpDat->real64Data();
	for (unsigned int i = 0 ; i < _LC->_timeStamp.size();i++)
	{
		timeDat[i] = _LC->_timeStamp[i];
		rateDat[i] = _LC->_rateCorr[i];
		rateErrDat[i] = _LC->_errRateCorr[i];
		fracExpDat[i] = _LC->_fracExp[i];
	}

	// Attributes
	*outTable->addAttribute("TELESCOP","XMM Mission" )=_telescope;
	*outTable->addAttribute( "INSTRUME",
				"Instrument name")= _instrument;
	*outTable->addAttribute("FILTER","Filter ID" )=_filter;
	*outTable->addAttribute("MJDREF","1998-01-01T00:00:00 (TT) expressed in MJD (TT)","d" )= _mjdref;
	*outTable->addAttribute( "HDUCLASS",
				"Format conforms to OGIP/GSFC conventions")= "OGIP";

	*outTable->addAttribute( "HDUCLAS1",
				"File contains a spectrum")= "LIGHTCURVE";

	std::string key;
	if (booleanParameter("withbkgset"))
		key = "TOTAL";
	else
		key = "NET";

	*outTable->addAttribute( "HDUCLAS2",
				"File contains gross rates")= key;

	*outTable->addAttribute( "HDUCLAS3",
				"Light curve stored as rates")= "RATE";
	*outTable->addAttribute( "HDUVERS1",
				"Version of format")= "1.1.0";


	//
	// Write DSS to fits file
	//
	*outTable->addAttribute("SLCTEXPR", "Filtering expression used by "
					     "evselect") = _dss.selectExpression();

	//
	// TIME Attributes
	//
	*outTable->addAttribute( "TIMESYS",
					_timeSysComment)= _timeSys;
	*outTable->addAttribute( "TIMEUNIT",
						"All times in s unless specified otherwise")= _timeUnit;
	*outTable->addAttribute( "TIMEREF",
						_timeRefComment)= _timeRef;
	*outTable->addAttribute( "TASSIGN",
						_timeAssignComment)= _timeAssign;
	*outTable->addAttribute( "CLOCKAPP",
						"Clock correction applied")= _timeClockApp;

	outTable << _dss;


	// Rate file specific keywords.

	*outTable->addAttribute( "TIMEDEL",
				"Light curve bin size")= _LC->_stride;
	*outTable->addAttribute( "TSTART",
				"Light curve start time")= _LC->_start;

	*outTable->addAttribute( "TSTOP",
				"Light curve end time")= _LC->_stop;
	*outTable->addAttribute( "BACKAPP", "Background correction applied?") = booleanParameter("withbkgset");
	*outTable->addAttribute( "DEADAPP", "Dead time correction applied?") = true;
	*outTable->addAttribute( "VIGNAPP", "Vignetting correction applied?") = booleanParameter("applyabsolutecorrections");

	delete cTimeDat;
	delete cRateDat;
	delete cRateErrDat;
	delete cFracExpDat;

	dataSetServer->close(out);

}


LightCurve* LightCurve::operator-(LightCurve* bkg)
{
	errstr << "Substracting background... "   << Sas::message(Msg::App,Msg::Sparse);
	//Check that the Lightcurve binning are aligned
	if (!this->getLC()->checkBinning(bkg->getLC()))
		errstr  << "Source and Background Lightcurve not aligned. Please, re-calculate "
				"the time series using the same start and stop time"<<  Sas::error("Binning");

	// Check that the backscale has been calculated and get the keywords.

	errstr << "Source BACKSCALE: "<< this->_spec->getBackscale() << Sas::message(Msg::App,Msg::Sparse);
	errstr << "Background BACKSCALE: "<< bkg->_spec->getBackscale() << Sas::message(Msg::App,Msg::Sparse);
	if ( (this->_spec->getBackscale() == 0) || ( bkg->_spec->getBackscale() == 0) )
		errstr  << "Source or Background backscale 0. "<<  Sas::error("ZerBackscale");

	double backScale = (double)this->_spec->getBackscale()/(double)bkg->_spec->getBackscale();

	this->getLC()->_rateCorr = this->getLC()->_rateCorr - ( bkg->getLC()->_rateCorr * backScale );

	//this->getLC()->_errRateCorr = sqrt( (double)  ( (this->getLC()->_errRateCorr * this->getLC()->_errRateCorr) + ( ( bkg->getLC()->_errRateCorr * backScale ) * ( bkg->getLC()->_errRateCorr * backScale ) ) ) );
	for (unsigned int i = 0 ; i < this->getLC()->_errRateCorr.size() ; i++)
	{
		this->getLC()->_errRateCorr[i] =
			sqrt( (double)  ( (this->getLC()->_errRateCorr[i] * this->getLC()->_errRateCorr[i]) + ( ( bkg->getLC()->_errRateCorr[i] * backScale ) * ( bkg->getLC()->_errRateCorr[i] * backScale ) ) ) );
	}




	errstr << "Done. "   << Sas::message(Msg::App,Msg::Sparse);
	return this;

}

